/*
 * This Source Code Form is subject to the terms of the MIT License.
 * If a copy of the MIT License was not distributed with this file,
 * you can obtain one at https://opensource.org/licenses/MIT
 */

/* entry point address for ELF */
ENTRY(isr_reset)

/* place stack at 64K SRAM */
_end_stack = 0x20010000;

/* Specify the memory areas */
MEMORY
{
  FLASH (rx)	: ORIGIN = 0x08000000, LENGTH = 64K
  RAM (xrw) 	: ORIGIN = 0x20000000, LENGTH = 128K
}

SECTIONS
{
 .text :
  {
    . = ALIGN(4);
    *(.isr_vector)
    *(.text)
    *(.text*)
    . = ALIGN(4);
    _end_text = .;
  } >FLASH

  .rodata :
  {
    . = ALIGN(8);
    *(.rodata)
    *(.rodata*)
    . = ALIGN(8);
  } >FLASH

  _stored_data = LOADADDR(.data);

  .data :
  {
    . = ALIGN(8);
    _start_data = .;
    *(.data)
    *(.data*)
    . = ALIGN(8);
    _end_data = .;
  } >RAM AT> FLASH

  . = ALIGN(8);
  .bss :
  {
    _start_bss = .;
    __bss_start__ = _start_bss;
    *(.bss)
    *(.bss*)
    *(COMMON)

    . = ALIGN(8);
    _end_bss = .;
    __bss_end__ = _end_bss;
    _start_heap = _end_bss;
  } >RAM

  /DISCARD/ :
  {
    libc.a ( * )
    libm.a ( * )
    libgcc.a ( * )
  }
}
